#include <iostream>
#include <vector>
#include <queue>
#include <algorithm>
#include <set>
#include <cassert>
using namespace std;
const int N = 50010;
const int EXIT_EVENT = 0;
const int QUERY_EVENT = 1;
#define TASK "queues"

typedef pair<int, int> pii;
typedef int TimeType;
const pii INVALID = make_pair(N, N);
const int K = 120;
const TimeType INF = 1e9;

struct Event {
    TimeType t;
    int type, x, y;
    bool operator < (const Event& other) const {
        return t > other.t || t == other.t && type > other.type || t == other.t && type == other.type && y > other.y;
    }
};
bool byQueue(const Event& a, const Event& b) {
    return a.x < b.x;
}

set<pii> lightPeople[N];
vector<pii> ti[N];
set<pii> heavyPeople[N];
int weight[N];
set<pii> manQueues[N];
priority_queue<Event> events;
bool isWait[N], isBusy[N];
int queryAns[N], who[N], queuePos[N];
Event tmpEvents[2 * N];
set<pair<TimeType, int> > waitedQueues[N];
TimeType lastExit[N];
TimeType T;
bool isLight[N];

int getTiByRecord(int queueId, int personId) {
    vector<pii>::iterator it = lower_bound(ti[queueId].begin(), ti[queueId].end(), make_pair(personId, -1));
    return it->second;
}

inline void addWaitQueue(int queueId) {
    isWait[queueId] = true;
    for (set<pii>::iterator it = heavyPeople[queueId].begin(); it != heavyPeople[queueId].end(); it++)
        waitedQueues[it->second].insert(make_pair(lastExit[queueId], queueId));
}

inline void removeWaitQueue(int queueId) {
    isWait[queueId] = false;
    for (set<pii>::iterator it = heavyPeople[queueId].begin(); it != heavyPeople[queueId].end(); it++)
        waitedQueues[it->second].erase(make_pair(lastExit[queueId], queueId));
}

inline void lightManEnter(int personId, int queueId) {
    //cout << "Enter in queue #" << queueId + 1 << " " << personId + 1 << endl;
    int ti = getTiByRecord(queueId, personId);
    events.push({T + ti, EXIT_EVENT, queueId, personId});
    if (isWait[queueId])
        removeWaitQueue(queueId);
    who[queueId] = personId;
    isBusy[personId]  = true;
    set<pii>& manQue = manQueues[personId];
    set<pii>::iterator itSv;
    for (set<pii>::iterator it = manQue.begin(); it != manQue.end(); it++) {
        if (it->first == queueId) {
            queuePos[queueId] = it->second + 1;
            itSv = it;
        }
        lightPeople[it->first].erase(make_pair(it->second, personId));
    }
    manQue.erase(itSv);
}

inline void lightManExit(int personId, int queueId) {
    who[queueId] = -1;
    isBusy[personId] = false;
    lastExit[queueId] = T;
    set<pii>& manQue = manQueues[personId];
    for (set<pii>::iterator it = manQue.begin(); it != manQue.end(); it++)
        lightPeople[it->first].insert(make_pair(it->second, personId));
}

inline void heavyManEnter(int personId, int queueId) {
    int ti = getTiByRecord(queueId, personId);
    events.push({T + ti, EXIT_EVENT, queueId, personId});
    if (isWait[queueId])
        removeWaitQueue(queueId);
    isBusy[personId] = true;
    who[queueId] = personId;
    for (set<pii>::iterator it = heavyPeople[queueId].begin(); it != heavyPeople[queueId].end(); it++)
        if (it->second == personId) {
            queuePos[queueId] = it->first + 1;
            heavyPeople[queueId].erase(it);
            break;
        }
}

inline void heavyManExit(int personId, int queueId) {
    isBusy[personId] = false;
    lastExit[queueId] = T;
    who[queueId] = -1;
}

inline pii getNextRecord(int queueId) {
    pii ret = INVALID;
    int position = queuePos[queueId];
    if (!lightPeople[queueId].empty()) {
        set<pii>::iterator it = lightPeople[queueId].lower_bound(make_pair(position, -1));
        if (it != lightPeople[queueId].end())
            ret = *it;
    }

    for (set<pii>::iterator it = heavyPeople[queueId].begin(); it != heavyPeople[queueId].end(); it++)
        if (it->first >= position && !isBusy[it->second]) {
            if (it->first < ret.first)
                ret = *it;
            break;
        }
    return ret;
}

int main() {
    freopen(TASK".in", "r", stdin);
    freopen(TASK".out", "w", stdout);
    //int l = clock();
    int n, m, k;
    cin >> n >> m >> k;
    for (int i = 0; i < m; ++i) {
        int ki;
        scanf("%d", &ki);
        ti[i] = vector<pii>(ki);
        for (int j = 0; j < ki; ++j) {
            int id, t;
            scanf("%d%d", &id, &t);
            ti[i][j] = make_pair(id - 1, t);
            manQueues[id - 1].insert({i, j});
            weight[id - 1]++;
        }
        events.push({0, EXIT_EVENT, i, n});
        sort(ti[i].begin(), ti[i].end());
    }

    for (int i = 0; i < k; ++i) {
        int id, t;
        scanf("%d%d", &id, &t);
        events.push({t, QUERY_EVENT, id - 1, i});
    }

    for (int i = 0; i <= n; ++i)
        isLight[i] = weight[i] <= K;

    for (int i = 0; i < n; ++i)
        if (!isLight[i]) {
            for (set<pii>::iterator q = manQueues[i].begin(); q != manQueues[i].end(); q++)
                heavyPeople[q->first].insert({q->second, i});
        } else {
            for (set<pii>::iterator q = manQueues[i].begin(); q != manQueues[i].end(); q++)
                lightPeople[q->first].insert(make_pair(q->second, i));
        }

    int szEvents, personId, queueId;
    int out = 0;
    while (!events.empty()) {
        szEvents = 1;
        tmpEvents[0] = events.top();
        events.pop();
        for (; !events.empty() && tmpEvents[0].t == events.top().t; szEvents++, events.pop())
            tmpEvents[szEvents] = events.top();
        T = tmpEvents[0].t;
        ++out;

        int exitEvents = 0;
        for (; exitEvents < szEvents && tmpEvents[exitEvents].type == EXIT_EVENT; ++exitEvents) {
            personId = tmpEvents[exitEvents].y;
            queueId = tmpEvents[exitEvents].x;
            if (isLight[personId])
                lightManExit(personId, queueId);
            else
                heavyManExit(personId, queueId);
        }

        for (int i = 0; i < exitEvents; ++i) {
            personId = tmpEvents[i].y;
            if (isLight[personId]) {
                pair<TimeType, int> found = make_pair(INF, N), tmp;
                int personPos;
                for (set<pii>::iterator it = manQueues[personId].begin(); it != manQueues[personId].end(); it++) {
                    tmp = make_pair(lastExit[it->first], it->first);
                    if (isWait[it->first] && tmp < found)
                        found = tmp;
                }
                if (found.second != N)
                    lightManEnter(personId, found.second);
            } else if (!waitedQueues[personId].empty())
                heavyManEnter(personId, waitedQueues[personId].begin()->second);
        }

        sort(tmpEvents, tmpEvents + exitEvents, byQueue);
        for (int i = 0; i < exitEvents; ++i) {
            queueId = tmpEvents[i].x;
            pii record = getNextRecord(queueId);
            if (record.first == N) {
                queuePos[queueId] = 0;
                record = getNextRecord(queueId);
            }

            if (record.first == N) {
                addWaitQueue(queueId);
                queuePos[queueId] = 0;
            } else {
                if (isLight[record.second])
                    lightManEnter(record.second, queueId);
                else
                    heavyManEnter(record.second, queueId);
            }
        }

        for (int i = exitEvents; i < szEvents; ++i)
            if (who[tmpEvents[i].x] == -1)
                queryAns[tmpEvents[i].y] = -1;
            else
                queryAns[tmpEvents[i].y] = who[tmpEvents[i].x] + 1;
    }
    for (int i = 0; i < k; ++i)
        printf("%d\n", queryAns[i]);
    //cerr << "time elapsed: " << 1.0 * (clock() - l) / CLOCKS_PER_SEC << endl;
    return 0;
}
