#include <iostream>
#include <vector>
#include <queue>
#include <algorithm>
#include <set>
using namespace std;
const int N = 50010;
const int EXIT_EVENT = 0;
const int QUERY_EVENT = 1;
#define TASK "queues"

typedef pair<int, int> pii;
typedef int TimeType;
const pii INVALID = make_pair(N, N);
const TimeType INF = 1e9;

struct Event {
    TimeType  t;
    int type, x, y;
    bool operator < (const Event& other) const {
        return t > other.t || t == other.t && type > other.type || t == other.t && type == other.type && y > other.y;
    }
};
bool byQueue(const Event& a, const Event& b) {
    return a.x < b.x;
}

vector<pii> queues[N];
vector<int> nxt[N], prv[N];
set<int> manQueues[N];
priority_queue<Event> events;
bool isBusy[N], isWait[N];
int queryAns[N], who[N], queuePos[N];
Event tmpEvents[2 * N];
TimeType lastExit[N];
TimeType T;
vector<pii> srt[N];

inline void manExit(int personId, int queueId) {
    who[queueId] = -1;
    isBusy[personId] = false;
    lastExit[queueId] = T;
}

inline void manEnter(int queueId) {
    isWait[queueId] = false;
    int pos = queuePos[queueId];
    pii cur = queues[queueId][pos];
    int personId = cur.first;
    who[queueId] = personId;
    isBusy[personId] = true;
    lastExit[queueId] = -1;
    events.push({T + cur.second, EXIT_EVENT, queueId, personId});
    manQueues[personId].erase(queueId);
    if (nxt[queueId][pos] == pos) queuePos[queueId] = -1;
    else {
        queuePos[queueId] = nxt[queueId][pos];
        prv[queueId][nxt[queueId][pos]] = prv[queueId][pos];
        nxt[queueId][prv[queueId][pos]] = nxt[queueId][pos];
    }
}

int getPosByRecord(int queueId, int personId) {
    vector<pii>::iterator it = lower_bound(srt[queueId].begin(), srt[queueId].end(), make_pair(personId, -1));
    return it->second;
}

int main() {
    freopen(TASK".in", "r", stdin);
    freopen(TASK".out", "w", stdout);
    int l = clock();
    int n, m, k;
    cin >> n >> m >> k;
    for (int i = 0; i < m; ++i) {
        int ki = N / m;
        scanf("%d", &ki);
        queues[i] = vector<pii>(ki);
        nxt[i] = vector<int>(ki, -1);
        prv[i] = vector<int>(ki, -1);
        srt[i] = vector<pii>(ki);
        for (int j = 0; j < ki; ++j) {
            int id, t;
            scanf("%d%d", &id, &t);
            manQueues[id - 1].insert(i);
            queues[i][j] = make_pair(id - 1, t);
            nxt[i][j] = (j + 1) % ki;
            prv[i][j] = (j + ki - 1) % ki;
            srt[i][j] = make_pair(id - 1, j);
        }
        sort(srt[i].begin(), srt[i].end());
        queuePos[i] = 0;
        events.push({0, EXIT_EVENT, i, n});
    }

    for (int i = 0; i < k; ++i) {
        int id, t;
        scanf("%d%d", &id, &t);
        events.push({t, QUERY_EVENT, id - 1, i});
    }

    int szEvents, personId, queueId;
    while (!events.empty()) {
        szEvents = 1;
        tmpEvents[0] = events.top();
        events.pop();
        for (; !events.empty() && tmpEvents[0].t == events.top().t; szEvents++, events.pop())
            tmpEvents[szEvents] = events.top();
        T = tmpEvents[0].t;

        int exitEvents = 0, cr, st;
        for (; exitEvents < szEvents && tmpEvents[exitEvents].type == EXIT_EVENT; ++exitEvents) {
            personId = tmpEvents[exitEvents].y;
            queueId = tmpEvents[exitEvents].x;
            manExit(personId, queueId);
        }

        for (int i = 0; i < exitEvents; ++i) {
            personId = tmpEvents[i].y;
            pair<TimeType, int> found = make_pair(INF, N), tmp;
            for (auto q: manQueues[personId]) {
                tmp = make_pair(lastExit[q], q);
                if (isWait[q] && tmp < found) found = tmp;
            }
            if (found.second != N) {
                int pos = getPosByRecord(found.second, personId);
                queuePos[found.second] = pos;
                manEnter(found.second);
            }
        }

        sort(tmpEvents, tmpEvents + exitEvents, byQueue);
        for (int i = 0; i < exitEvents; ++i) {
            queueId = tmpEvents[i].x;
            cr = queuePos[queueId];
            if (cr == -1)
                continue;
            st = cr;
            int steps = 0;
            do {
                if (!isBusy[queues[queueId][cr].first]) {
                    manEnter(queueId);
                    break;
                }
                cr = nxt[queueId][cr];
                queuePos[queueId] = cr;
                ++steps;
            } while (cr != st);
            if (st == cr && steps > 0)
                isWait[queueId] = true;
        }

        for (int i = exitEvents; i < szEvents; ++i)
            if (who[tmpEvents[i].x] == -1)
                queryAns[tmpEvents[i].y] = -1;
            else
                queryAns[tmpEvents[i].y] = who[tmpEvents[i].x] + 1;
    }
    for (int i = 0; i < k; ++i)
        printf("%d\n", queryAns[i]);
    cerr << "time elapsed: " << 1.0 * (clock() - l) / CLOCKS_PER_SEC << endl;
    return 0;
}
/*
1 1 3
1 1 2
1 1
1 2
1 3

 */
