import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;
import java.util.function.Consumer;
import java.util.function.Function;

public class TestGen {
    private static final int MAX_QUERY_T = 1_000_000_000;
    Random random = new Random(0xBEE);
    public static final int MAX_N = 50000;
    public static final int MAX_T = 10000;

    int testCount;
    void print(Consumer<PrintWriter> consumer, String comment) throws IOException {
        PrintWriter out = new PrintWriter(String.format("../tests/%02d", ++testCount));
        System.out.println("Generating test #" + testCount + ": " + comment);
        consumer.accept(out);
        out.close();
    }

    private boolean isFirstIdT;

    private void printIdT(int id, int tm, PrintWriter out) {
        if (isFirstIdT)
            out.print(id + " " + tm);
        else
            out.print(" " + id + " " + tm);
        isFirstIdT = false;
    }

    private void printKi(int ki, PrintWriter out) {
        out.print(ki + " ");
        isFirstIdT = true;
    }

    private void printNMK(int n, int m, int k, PrintWriter out) {
        out.println(n + " " + m + " " + k);
    }

    private void printQuery(int id, int t, PrintWriter out) {
        out.println(id + " " + t);
    }

    void genSamples() throws IOException {
        printSample(
                "5 3 4\n" +
                "3 1 2 2 4 4 1\n" +
                "2 1 3 5 6\n" +
                "2 5 3 1 3\n" +
                "3 1\n1 2\n2 4\n3 10\n", "sample1");

        printSample(
                "5 4 5\n" +
                "5 1 1 2 1 3 1 4 1 5 1\n" +
                "1 1 10\n" +
                "2 1 3 2 4\n" +
                "3 1 1 2 2 3 3\n" +
                "1 1\n2 2\n3 15\n4 5\n4 7\n", "sample2");
    }

    private void printSample(String s, String comment) throws IOException {
        PrintWriter out = new PrintWriter(String.format("../tests/%02d", ++testCount));
        System.out.println("Generating test #" + testCount + ": " + comment);
        out.print(s);
        out.close();
    }

    @FunctionalInterface
    interface CycleGen {
        int gen(int i, int j, int n, int m);
    }

    Function<Integer, List<Integer>> revPerm = n -> {
        List<Integer> perm = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            perm.add(n - i);
        return perm;
    };

    Function<Integer, List<Integer>> uniform = m->{
        List<Integer> list = new ArrayList<>();
        for (int i = 0; i < m; ++i)
            list.add(MAX_N / m);
        return list;
    };

    Function<Integer, List<Integer>> noUniform = m->{
        List<Integer> comb = randComb(MAX_N, m - 1);
        List<Integer> ret = new ArrayList<>();
        ret.add(comb.get(0));
        for (int i = 0; i + 1 < comb.size(); ++i)
            ret.add(comb.get(i + 1) - comb.get(i));
        ret.add(MAX_N - comb.get(comb.size() - 1));
        return ret;
    };

    private List<Integer> randComb(int n, int k) {
        List<Integer> perm = randPerm.apply(n);
        perm = perm.subList(0, k);
        Collections.sort(perm);
        return perm;
    }

    Function<Integer, List<Integer>> randPerm = n -> genDisposition(n, n);

    void run() throws IOException {
        new File("../tests").mkdir();
        genSamples();

        genSameTime(MAX_N, MAX_N, MAX_N);
        genSameTime(MAX_N, MAX_N / 2, MAX_N);
        genSameTime(MAX_N, MAX_N / 5, MAX_N);
        genSameTime(MAX_N, MAX_N / 10, MAX_N);
        genSameTime(MAX_N, MAX_N / 300, MAX_N);
        genSameTime(MAX_N, MAX_N / 1000, MAX_N);

        int[] BLOCKS = {2, 30, 50, 80, 100, 150, 200, 250, 300, 350, 400, 450, 500, 1000, MAX_N / 25, MAX_N / 10, MAX_N / 4, MAX_N / 2};
        for (Integer block: BLOCKS) {
            genCyclic(MAX_N, block, MAX_N, (i, j, n, m)->(i + j) % n + 1, "cyclic (i+j)");
            genCyclic(MAX_N, block, MAX_N, (i, j, n, m)->j % n + 1, "cyclic j");
        }

        genOneBig(MAX_N / 2, MAX_N / 2 + 1, MAX_N, revPerm, "one big: rev perm");
        genOneBig(MAX_N / 2, MAX_N / 2 + 1, MAX_N, randPerm, "one big: rand perm");
        genOneBig(MAX_N / 2, MAX_N / 2 + 1, MAX_N, randPerm, "one big: rand perm");
        genOneQueue(MAX_N, MAX_N);
        genOneQueue(MAX_N, MAX_N);

        genRandBySize(MAX_N, MAX_N / 2, MAX_N, uniform, MAX_T, MAX_QUERY_T, "gen rand by size: uniform");
        genRandBySize(2, MAX_N / 2, MAX_N, uniform, MAX_T, MAX_QUERY_T, "gen rand by size: uniform");
        genRandBySize(5, MAX_N / 5, MAX_N, uniform, MAX_T, MAX_QUERY_T, "gen rand by size: uniform");
        genRandBySize(10, MAX_N / 5, MAX_N, uniform, MAX_T, MAX_QUERY_T, "gen rand by size: uniform");
        genRandBySize(50, MAX_N / 50, MAX_N, uniform, MAX_T, MAX_QUERY_T, "gen rand by size: uniform");
        genRandBySize(MAX_N, 2, MAX_N, uniform, 100, 1000, "gen rand by size: uniform");
        genRandBySize(MAX_N, 30, MAX_N, uniform, 10, 10000, "gen rand by size: uniform");
        genRandBySize(MAX_N, 50, MAX_N, uniform, 1000, MAX_QUERY_T, "gen rand by size: uniform");
        genRandBySize(MAX_N, 100, MAX_N, uniform, MAX_T, MAX_T, "gen rand by size: uniform");
        genRandBySize(MAX_N, 200, MAX_N, uniform, MAX_T, MAX_T, "gen rand by size: uniform");
        genRandBySize(MAX_N, 200, MAX_N, uniform, MAX_T, MAX_QUERY_T, "gen rand by size: uniform");
        genRandBySize(MAX_N, 300, MAX_N, uniform, 1, MAX_N, "gen rand by size: uniform");
        genRandBySize(MAX_N, 300, MAX_N, uniform, 100, 10000, "gen rand by size: uniform");
        genRandBySize(MAX_N, 500, MAX_N, uniform, MAX_T, MAX_QUERY_T, "gen rand by size: uniform");
        genRandBySize(MAX_N, 5000, MAX_N, uniform, MAX_T, MAX_QUERY_T, "gen rand by size: uniform");
        int[] MS = {5, 10, 20, 40, 100, 200, 300, 500};
        for (Integer ms : MS)
            genRandBySize(MAX_N, ms, MAX_N, noUniform, MAX_T, MAX_QUERY_T, "gen rand by size: noUniform");

    }

    private void genSameTime(int n, int m, int k) throws IOException {
        int t = randInt(MAX_T);
        print(out-> {
            printNMK(n, m, k, out);
            for (int i = 0; i < m; ++i) {
                int ki = MAX_N / m;
                printKi(ki, out);
                for (int j = 0; j < ki; ++j)
                    printIdT((int)((i + 1L * (n / ki) * j) % n + 1), t, out);
                out.println();
            }
            for (int i = 0; i < k; ++i)
                printQuery(i % m + 1, t, out);
        }, "gen same time n = " + n + " m = " + m + " k = " + k + " t = " + t);
    }

    void genOneQueue(int n, int k) throws IOException{
        print(out->{
            printNMK(   n, 1, k, out);
            List<Integer> perm = randPerm.apply(n);
            printKi(n, out);
            for (int i = 0; i < n; ++i)
                printIdT(perm.get(i), randInt(MAX_T), out);
            out.println();
            for (int i = 1; i <= k; ++i)
                printQuery(1, randInt(MAX_QUERY_T), out);
        }, "one queue, n = " + n + ", k = " + k);
    }

    void genCyclic(int n, int m, int k, CycleGen generator, String comment) throws IOException {
        print(out -> {
            int ki = MAX_N / m;
            final int T = (random.nextBoolean() ? 100 : 1000);
            printNMK(n, m, k, out);
            for (int i = 0; i < m; ++i) {
                printKi(ki, out);
                for (int j = 0; j < ki; ++j) {
                    int id = generator.gen(i, j, n, m);
                    int tm = randInt(T);
                    printIdT(id, tm, out);
                }
                out.println();
            }
            for (int i = 0; i < k; ++i)
                printQuery(i % m + 1, randInt(n), out);
        }, comment + " n = " + n + " m = " + m + " k = " + k);
    }

    void genOneBig(int n, int m, int k,  Function<Integer, List<Integer>> permGenerator, String comment) throws IOException {
        print(out -> {
            printNMK(n, m, k, out);
            for (int i = 0; i < m; ++i) {
                if (i + 1 < m) {
                    printKi(1, out);
                    printIdT(i + 1, (i + 1) % MAX_T + 1, out);
                    out.println();
                } else {
                    printKi(n, out);
                    List<Integer> perm = permGenerator.apply(n);
                    for (Integer x : perm)
                        printIdT(x, 1, out);
                    out.println();
                }
            }
            for (int i = 0; i < k; ++i)
                printQuery(m, i + 1, out);
        }, comment + " n = " + n + " m = " + " k = " + k);
    }

    void genRandBySize(int n, int m, int k, Function<Integer, List<Integer>> listGen, int maxVal, int maxValQ, String comment) throws IOException {
        print(out -> {
            printNMK(n, m, k, out);
            List<Integer> list = listGen.apply(m);
            for (int i = 0; i < m; ++i) {
                int ki = list.get(i);
                printKi(ki, out);
                List<Integer> despotition = genDisposition(n, ki);
                for (int j = 0; j < ki; ++j)
                    printIdT(despotition.get(j), randInt(maxVal), out);
                out.println();
            }
            for (int i = 0; i < k; ++i)
                printQuery(randInt(m), randInt(maxValQ), out);
        }, comment + " n = " + n + " m = " + m + " k = " + k);
    }

    List<Integer> genDisposition(int n, int k) {
        if (k > n)
            throw new IllegalArgumentException("k > n");

        if (k > n / 2) {
            List<Integer> p = new ArrayList<>();
            for (int i = 0; i < n; ++i)
                p.add(i + 1);
            shuffle(p);
            return p.subList(0, k);
        } else {
            List<Integer> p = new ArrayList<>();
            Set<Integer> s = new HashSet<>();
            while (s.size() < k)
                s.add(randInt(n));
            for (Integer x : s)
                p.add(x);
            return p;
        }
    }

    int randInt(int bound) {
        return random.nextInt(bound) + 1;
    }

    void shuffle(List<?> list) {
        Collections.shuffle(list, random);
    }

    void genTest(int t) {

    }

    public static void main(String[] args) throws IOException {
        if (args.length == 1)
            new TestGen().genTest(Integer.parseInt(args[0]));
        else
            new TestGen().run();
    }
}
